<?php

    /**
     * Класс подсветки языков программирования
     *
     * @author Zmi
     */
    class SyntaxHighlighter
    {
        private $syntaxHighlighterNames = array(
                                                    'bash'              => 'shBrushBash',
                                                    'cpp'               => 'shBrushCpp',
                                                    'cshshlfilesarp'    => 'shBrushCSharp',
                                                    'css'               => 'shBrushCss',
                                                    'delphi'            => 'shBrushDelphi',
                                                    'diff'              => 'shBrushDiff',
                                                    'gvoovy'            => 'shBrushGroovy',
                                                    'java'              => 'shBrushJava',
                                                    'jscript'           => 'shBrushJScript',
                                                    'php'               => 'shBrushPhp',
                                                    'plain'             => 'shBrushPlain',
                                                    'python'            => 'shBrushPython',
                                                    'ruby'              => 'shBrushRuby',
                                                    'scala'             => 'shBrushScala',
                                                    'sql'               => 'shBrushSql',
                                                    'vb'                => 'shBrushVb',
                                                    'xml'               => 'shBrushXml'
                                                );

        private static $aSyntaxHighlighter  = array(
                                                      'bash'    => array('bash'),
                                                      'cpp'     => array('cpp', 'c++', 'c', 'cplusplus', 'c-plusplus'),
                                                      'csharp'  => array('csharp', 'c#', 'c-sharp'),
                                                      'css'     => array('css'),
                                                      'delphi'  => array('delphi', 'pascal'),
                                                      'diff'    => array('diff'),
                                                      'gvoovy'  => array('gvoovy'),
                                                      'java'    => array('java'),
                                                      'jscript' => array('jscript', 'js', 'javascript'),
                                                      'php'     => array('php'),
                                                      'plain'   => array('plain'),
                                                      'python'  => array('python'),
                                                      'ruby'    => array('ruby'),
                                                      'scala'   => array('scala'),
                                                      'sql'     => array('sql', 'mysql'),
                                                      'vb'      => array('vb', 'visualbasic', 'basic', 'visual-basic'),
                                                      'xml'     => array('xml', 'html')
                                                   );

        public static function TranslateLangForSyntaxHighlighter($lang)
        {
            $lang = strtolower($lang);
            $ret = 'bash';
            foreach (self::$aSyntaxHighlighter as $name => $variants)
            {
                foreach ($variants as $v)
                {
                    if ($lang == $v)
                    {
                        $ret = $name;
                        break(2);
                    }
                }
            }
            return $ret;
        }

        public function GetIncFileFromName($name)
        {
            $name = self::TranslateLangForSyntaxHighlighter($name);
            return isset($this->syntaxHighlighterNames[$name]) ? $this->syntaxHighlighterNames[$name] : '';
        }

        public static function SetSyntaxHighlinerLang($m)
        {
            global $config;
            require_once BASEPATH . 'lib/InputClean.php';

            $charset = $config['charset'];
            $input   = new InputClean($charset);

            $lang = isset($m[1]) ? htmlentities($m[1], ENT_QUOTES) : '';
            $lang = $input->xss_clean($lang);

            $text = isset($m[2]) ? $m[2] : '';
            $text = $input->xss_clean($text);

            $lang = self::TranslateLangForSyntaxHighlighter($lang);
            return "<pre class='brush: {$lang};'>{$text}</pre>";
        }

        public function PrepareContent($c)
        {
            if (strpos($c, '<!-- list languages -->') === false)
            {
                return $c;
            }

            // Регуляка для выбора всех [code=ЯЗЫК]...[/code]
            $regExpGetAllCode = "~\[code=(.*?)\](.*?)\[/code\]~uis";

            // Получаем список всех используемых языков что бы потом подключить js файлы с их подстветкой
            preg_match_all($regExpGetAllCode, $c, $m);
            $arrIncLangs = isset($m[1]) ? array_unique($m[1]) : array();

            if (!empty($arrIncLangs))
            {
                $incScripts = '';
                foreach ($arrIncLangs as $lang)
                {
                    $addr = Root('i/js/dev/syntaxhighlighter/' . $this->GetIncFileFromName($lang) . '.js');
                    $incScripts .= "<script type='text/javascript' src='{$addr}'></script>";
                }
                $c = str_replace('<!-- list languages -->', $incScripts, $c);
            }

            // Заменяем все [code] ... [/code] на <pre> ... </pre>
            return preg_replace_callback($regExpGetAllCode, array(__CLASS__, "SetSyntaxHighlinerLang"), $c);
        }
    };
?>